%GSP_DEMO1_SPECTRUM_ADAPTED_FILTERS
%
%   Url: http://gspbox.sourceforge.net/doc/demo/gsp_demo1_spectrum_adapted_filters.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

close all;
clear all; 
set(0,'DefaultFigureWindowStyle','docked');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up graph and plot it 
graph_type='sensor';

switch graph_type;
    case 'path'
        param.N=64;
        G=gsp_create_graph('path',param);
    case 'ring'
        param.N=64;
        G=gsp_create_graph('ring',param);
    case 'sensor'
        param.N=64;
        G=gsp_create_graph('sensor',param);
    case 'comet'
        param.N=64;
        param.k=30;
        G=gsp_create_graph('comet',param);
    otherwise
        error('Unknown graph type');
end

gsp_plot_graph(G);

% Compute the full spectrum
if G.N < 3500
    G=gsp_full_eigen(G);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct and plot different systems of graph spectral filters
num_filters=8;
plot_param.colors_rgb=[0,0,1;1 .5 0;0 1 0; 1 0 1;0 1 1; 1 1 0;.5 .5 .5;1,0,0];
plot_param.lambda_full=G.E;
plot_param.lambda_highlights=[0,G.lmax];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 1: Only adapted to the length of the spectrum

% 1.1. Uniform translates
[uniform_filters]=gsp_filter_design('uniform_translates',num_filters,G.lmax);
plot_param.title = 'Uniform Translates Filter Bank';
[~,uniform_test_sum]=gsp_plot_filters(uniform_filters,G.lmax,plot_param);
uniform_frame_bounds=[min(uniform_test_sum),max(uniform_test_sum)]

% 1.2. SGWT (Hammond et al., 2011)
[sgwt_filters]=gsp_filter_design('sgwt',num_filters,G.lmax);
plot_param.title = 'SGWT Filter Bank (Spline-Based)';
[~,sgwt_test_sum]=gsp_plot_filters(sgwt_filters,G.lmax,plot_param);
sgwt_frame_bounds=[min(sgwt_test_sum),max(sgwt_test_sum)]

% 1.3. Tight Meyer frame (Leonardi and Van De Ville, 2011 and 2013)
[sgwt_meyer_filters]=gsp_filter_design('sgwt_meyer',num_filters,G.lmax);
plot_param.title = 'Tight Frame of Meyer Kernels';
[~,sgwt_meyer_test_sum]=gsp_plot_filters(sgwt_meyer_filters,G.lmax,plot_param);
sgwt_meyer_frame_bounds=[min(sgwt_meyer_test_sum),max(sgwt_meyer_test_sum)]

% 1.4. Tight wavelet frame via log warping
[log_warp_filters]=gsp_filter_design('warped_translates',num_filters,G.lmax);
plot_param.title = 'Tight Wavelet Frame Based on Log-Warped Uniform Translates';
[~,log_warp_test_sum]=gsp_plot_filters(log_warp_filters,G.lmax,plot_param);
log_warp_frame_bounds=[min(log_warp_test_sum),max(log_warp_test_sum)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 2: Adapted to the length of the spectrum and the maximum degree

% 2.1. Tight Meyer frame adapted to max degree (Leonardi and Van De Ville, 2013)
meyer_degree_param.max_degree=max(max(G.D));
[meyer_degree_filters]=gsp_filter_design('sgwt_meyer_degree',num_filters,G.lmax,meyer_degree_param);
plot_param.title = 'Tight Frame of Meyer Kernels Adapted to the Maximum Degree';
[~,meyer_degree_test_sum]=gsp_plot_filters(meyer_degree_filters,G.lmax,plot_param);
meyer_degree_frame_bounds=[min(meyer_degree_test_sum),max(meyer_degree_test_sum)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 3: Adapted to the distribution of Laplacian eigenvalues

% 3.1 Spectrum-adapted vertex-frequency filters 
warp_function_type = 'approx';

switch warp_function_type
    case 'exact'
        tol=1e-8;
        [unique_E unique_E_inds]=unique(round(G.E*1/tol)*tol);
        vertex_freq_param.approx_spectrum.x=unique_E;
        vertex_freq_param.approx_spectrum.y=(unique_E_inds-1)/(G.N-1);  
        vertex_freq_param.warp_function_type='mono_cubic';
        vertex_freq_param.upper_bound_translates=1;
        [vertex_freq_filters,warp_fn]=gsp_filter_design('warped_translates',num_filters,G.lmax,vertex_freq_param);
    case 'subset'
        approx_spectrum_inds=1:9:64;
        N_bar=length(approx_spectrum_inds);
        tol=1e-8;
        [unique_subset_E unique_subset_E_inds]=unique(round(G.E(approx_spectrum_inds)*1/tol)*tol);
        vertex_freq_param.approx_spectrum.x=unique_subset_E;
        vertex_freq_param.approx_spectrum.y=(unique_subset_E_inds-1)/(N_bar-1);  
        vertex_freq_param.warp_function_type='mono_cubic';
        vertex_freq_param.upper_bound_translates=1;
        [vertex_freq_filters,warp_fn]=gsp_filter_design('warped_translates',num_filters,G.lmax,vertex_freq_param);
    case 'approx'
        G=gsp_spectrum_cdf_approx(G);
        warp_fn=G.spectral_warp_fn;
        vertex_freq_param.warp_function_type='custom';
        vertex_freq_param.warp_function=warp_fn;
        vertex_freq_param.upper_bound_translates=1;
        [vertex_freq_filters]=gsp_filter_design('warped_translates',num_filters,G.lmax,vertex_freq_param);
    otherwise
        error('unknown warp function type');
end

gsp_plot_warp_fn(warp_fn,[0,G.lmax]);
title('Warping Function to Approximate Cumulative Spectral Density Function','FontSize',18);

plot_param.title='Spectrum-Adapted Tight Vertex-Frequency Frame';
[~,vertex_freq_test_sum]=gsp_plot_filters(vertex_freq_filters,G.lmax,plot_param);
vertex_freq_frame_bounds=[min(vertex_freq_test_sum),max(vertex_freq_test_sum)]

% 3.2 Spectrum-adapted tight wavelet filters
warped_wavelet_param.warp_function_type='log_plus';
warped_wavelet_param.warp_function=@(s) G.lmax*warp_fn(s);
[warped_wavelet_filters,warped_wavelet_warp_function]=gsp_filter_design('warped_translates',num_filters,G.lmax,warped_wavelet_param);
plot_param.title='Spectrum-Adapted Tight Wavelet Frame';
[~,warped_wavelet_test_sum]=gsp_plot_filters(warped_wavelet_filters,G.lmax,plot_param);
warped_wavelet_frame_bounds=[min(warped_wavelet_test_sum),max(warped_wavelet_test_sum)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 4: Adapted to a subset approximation of the spectrum

% 4.1 Spectrum-adapted lapped filters
approx_spectrum_inds=1:9:64;
lapped_param.approx_spectrum=G.E(approx_spectrum_inds);
[lapped_filters]=gsp_filter_design('lapped_kernels',num_filters,G.lmax,lapped_param);
plot_param.title = 'Spectrum-Adapted Lapped Filters';
plot_param.lambda_highlights=lapped_param.approx_spectrum;
[~,lapped_test_sum]=gsp_plot_filters(lapped_filters,G.lmax,plot_param);
lapped_frame_bounds=[min(lapped_test_sum),max(lapped_test_sum)]
