%GSP_DEMO2_RANDOM_GRAPH_FILTERS
%
%   Url: http://gspbox.sourceforge.net/doc/demo/gsp_demo2_random_graph_filters.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

close all;
clear all; 
set(0,'DefaultFigureWindowStyle','docked');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up graph and plot it 
graph_type='erdos_renyi_normalized';

switch graph_type;
    case 'random_regular'
        param.N=500;
        param.r=3;
        G=gsp_create_graph('random_regular',param);
        lambda_upper=2*param.r;
    case 'erdos_renyi_combinatorial'
        param.N=3000;
        param.p=.05;
        param.lap_type='combinatorial';
        G=gsp_create_graph('erdos_renyi',param);
        lambda_upper=param.p*param.N+4*sqrt(param.p*(1-param.p)*param.N);
        plot_param.dlambda=.2;
    case 'erdos_renyi_normalized'
        param.N=3000;
        param.p=.05;
        param.lap_type='normalized';
        G=gsp_create_graph('erdos_renyi',param);
        lambda_upper=2;
    otherwise
        error('Unknown graph type');
end

% If the graph is small, compute the full spectrum and use the Laplacian eigenvectors to generate coordinates
if G.N < 3500
    G=gsp_full_eigen(G);
    if G.N < 1000
        G.coords=G.U(:,2:3);
        gsp_plot_graph(G); % Will not plot edges if there are too many in the graph to show
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct and plot different systems of graph spectral filters
num_filters=8;
plot_param.colors_rgb=[0,0,1;1 .5 0;0 1 0; 1 0 1;0 1 1; 1 1 0;.5 .5 .5;1,0,0];
plot_param.lambda_full=G.E;
plot_param.lambda_highlights=[0,lambda_upper];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 1: Only adapted to the length of the spectrum

% 1.1. Uniform translates
[uniform_filters]=gsp_filter_design('uniform_translates',num_filters,lambda_upper);
plot_param.title = 'Uniform Translates Filter Bank';
[~,uniform_test_sum]=gsp_plot_filters(uniform_filters,lambda_upper,plot_param);
uniform_frame_bounds=[min(uniform_test_sum),max(uniform_test_sum)]

% 1.2. SGWT (Hammond et al., 2011)
[sgwt_filters]=gsp_filter_design('sgwt',num_filters,lambda_upper);
plot_param.title = 'SGWT Filter Bank (Spline-Based)';
[~,sgwt_test_sum]=gsp_plot_filters(sgwt_filters,lambda_upper,plot_param);
sgwt_frame_bounds=[min(sgwt_test_sum),max(sgwt_test_sum)]

% 1.3. Tight Meyer frame (Leonardi and Van De Ville, 2011 and 2013)
[sgwt_meyer_filters]=gsp_filter_design('sgwt_meyer',num_filters,lambda_upper);
plot_param.title = 'Tight Frame of Meyer Kernels';
[~,sgwt_meyer_test_sum]=gsp_plot_filters(sgwt_meyer_filters,lambda_upper,plot_param);
sgwt_meyer_frame_bounds=[min(sgwt_meyer_test_sum),max(sgwt_meyer_test_sum)]

% 1.4. Tight wavelet frame via log warping
[log_warp_filters]=gsp_filter_design('warped_translates',num_filters,lambda_upper);
plot_param.title = 'Tight Wavelet Frame Based on Log-Warped Uniform Translates';
[~,log_warp_test_sum]=gsp_plot_filters(log_warp_filters,lambda_upper,plot_param);
log_warp_frame_bounds=[min(log_warp_test_sum),max(log_warp_test_sum)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 2: Adapted to the length of the spectrum and the maximum degree

% 2.1. Tight Meyer frame adapted to max degree (Leonardi and Van De Ville, 2013)
meyer_degree_param.max_degree=max(max(G.D));
[meyer_degree_filters]=gsp_filter_design('sgwt_meyer_degree',num_filters,lambda_upper,meyer_degree_param);
adapted_plot_param=plot_param;
adapted_plot_param.title = 'Tight Frame of Meyer Kernels Adapted to the Maximum Degree';
adapted_plot_param.yrange=[0,1.25];
[~,meyer_degree_test_sum]=gsp_plot_filters(meyer_degree_filters,lambda_upper,adapted_plot_param);
meyer_degree_frame_bounds=[min(meyer_degree_test_sum),max(meyer_degree_test_sum)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Section 3: Adapted to the limiting distribution of Laplacian eigenvalues
% (but not to the specific realization of the random graph or its
% eigenvalues)

% 3.1 Spectrum-adapted vertex-frequency filters 

switch graph_type;
    case 'random_regular'
        [vertex_freq_filters,warp_fn]=gsp_filter_design('random_regular_lap',num_filters,lambda_upper,param);
    case 'erdos_renyi_combinatorial'
        [vertex_freq_filters,warp_fn]=gsp_filter_design('erdos_renyi_lap',num_filters,lambda_upper,param);
    case 'erdos_renyi_normalized'
        [vertex_freq_filters,warp_fn]=gsp_filter_design('erdos_renyi_norm_lap',num_filters,lambda_upper,param);
    otherwise
        error('Unknown graph type');
end

gsp_plot_warp_fn(warp_fn,lambda_upper);
title('Warping Function Based on the Limiting Cumulative Spectral Density Function','FontSize',18);

plot_param.title='Spectrum-Adapted Tight Vertex-Frequency Frame';
[~,vertex_freq_test_sum]=gsp_plot_filters(vertex_freq_filters,lambda_upper,plot_param);
vertex_freq_frame_bounds=[min(vertex_freq_test_sum),max(vertex_freq_test_sum)]

% 3.2 Spectrum-adapted tight wavelet filters
warped_wavelet_param.warp_function_type='log_plus';
warped_wavelet_param.warp_function=@(s) lambda_upper*warp_fn(s);
[warped_wavelet_filters,warped_wavelet_warp_function]=gsp_filter_design('warped_translates',num_filters,lambda_upper,warped_wavelet_param);
plot_param.title='Spectrum-Adapted Tight Wavelet Frame';
[~,warped_wavelet_test_sum]=gsp_plot_filters(warped_wavelet_filters,lambda_upper,plot_param);
warped_wavelet_frame_bounds=[min(warped_wavelet_test_sum),max(warped_wavelet_test_sum)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compare actual density of Laplacian eigenvalues to the limiting density
switch graph_type;
    case 'random_regular'
        xlen=.01;
        xtic=1;
        ytic=.05;
        yrange=[0 .3];
        lap_density = @(y) gsp_random_regular_density(y,param.r);        
    case 'erdos_renyi_combinatorial'
        xlen=.05;
        xtic=20;
        ytic=.005;
        yrange=[0 .025];
        lap_density = @(y) gsp_erdos_renyi_density(y,param.N,param.p);
     case 'erdos_renyi_normalized'
        xlen=.01;
        xtic=.25;
        ytic=1;
        yrange=[0 4.25];
        lap_density = @(y) gsp_erdos_renyi_density_normalized(y,param.N,param.p);
    otherwise
        error('Unknown graph type');
end

figure
hold on;
num_buckets=70;
histnorm(G.E,num_buckets);
xx=0:xlen:lambda_upper;
plot(xx,lap_density(xx),'r','LineWidth',4);
box on;
set(gca,'FontSize',24);
xlim([0 lambda_upper]);
ylim(yrange);
set(gca,'XTick',0:xtic:lambda_upper);
set(gca,'YTick',0:ytic:yrange(2));
set(gca,'XMinorTick','on','YMinorTick','on');

