function [G]=gsp_create_graph(graph_type,varargin)
%GSP_CREATE_GRAPH  Initialize a graph
%   Usage:  G=gsp_create_graph(graph_type);
%           G=gsp_create_graph(graph_type,param);
%
%   Input parameters:
%         graph_type : String containing graph type.
%         param      : Optional variable containing additional parameters.
%   Output parameters:
%         G          : Graph structure.
%   Supported graph types:
%         custom     : Requires the adjacency matrix to be entered in param.W. 
%         path       :
%         comet      :
%         sensor     :
%         minnesota  :
%         airfoil    :
%         ring       :
%         swiss      :
%   Additional input parameters:
%         param.W         : Weighted adjacency matrix (required for 'custom' graph).
%         param.N         : Number of vertices.
%         param.lap_type  : Options include 'combinatorial' (default), 'normalized', 'none'. 
%
%   'gsp_create_graph(graph_type)' initializes a graph structure containing
%   the graph type (G.graph_type), the weighted adjacency matrix (G.W), 
%   the unweighted adjacency matrix (G.A), the number of vertices (G.N), 
%   the degree matrix (G.D), the graph Laplacian type (G.lap_type), the 
%   graph Laplacian (G.L), the plotting coordinates (G.coords), the 
%   coordinate limits (G.coord_limits), the edge width (G.edge_width), the 
%   edge color (G.edge_color), the edge style (G.edge_style), and the 
%   vertex size (G_vertex_size).
% 
%   Insert definitions of degree, laplacian, etc.
%
%   See also:  
%
%   Demos:  
% 
%   References:
%     F. R. K. Chung. Spectral Graph Theory. Vol. 92 of the CBMS Regional
%     Conference Series in Mathematics, American Mathematical Society, 1997.
%     
%     
%
%
%   Url: http://gspbox.sourceforge.net/doc/graphs/gsp_create_graph.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
% Read input parameters
if nargin>1
    param=varargin{1};
else
    param=0;
end

% Create weighted adjancency matrix
switch graph_type
    case 'custom'
        if ~isfield(param,'W')
            error('Weighted adjacency matrix not specified for custom graph');
        else
            G.W=param.W;
        end
        G.N=size(G.W,1);
    case 'path' 
        if ~isfield(param,'N')
            N=64;
        else
            N=param.N;
        end
        G=gsp_create_path_graph(N);
    case 'comet'    
        if ~isfield(param,'N')
            N=64;
        else
            N=param.N;
        end
        if ~isfield(param,'k')
            k=30;
        else
            k=param.k;
        end
        G=gsp_create_comet_graph(N,k);
    case 'sensor' 
        if ~isfield(param,'N')
            N=64;
        else
            N=param.N;
        end
        G=gsp_create_sensor_network(N);
    case 'minnesota'
        G=gsp_create_minnesota_graph;
    case 'airfoil'
        G=gsp_create_airfoil_graph;
    case 'ring'
        if ~isfield(param,'N')
            N=8;
        else
            N=param.N;
        end
        if ~isfield(param,'k')
            k=1;
        else
            k=param.k;
        end
        G=gsp_create_ring_graph(N,k);
    case 'swiss'
        if ~isfield(param,'N')
            N=2000;
        else
            N=param.N;
        end
        if ~isfield(param,'s')
            s=.1;
        else
            s=param.s;
        end
        if ~isfield(param,'thresh')
            thresh=.1;
        else
            thresh=param.thresh;
        end
        if ~isfield(param,'rand_state')
            rand_state=45;
        else
            rand_state=param.rand_state;
        end
        G=gsp_create_swiss_roll(N,s,thresh,rand_state);
    case 'erdos_renyi' 
        if ~isfield(param,'N')
            N=3000;
        else
            N=param.N;
        end
        if ~isfield(param,'p')
            p=.05;
        else
            p=param.p;
        end
        G=gsp_create_erdos_renyi(N,p);
    case 'random_regular' 
        if ~isfield(param,'N')
            N=3000;
        else
            N=param.N;
        end
        if ~isfield(param,'r')
            r=3;
        else
            r=param.r;
        end
        G.N=N;
        G.W=createRandRegGraph(N,r); % Use downloaded code 'createRandRegGraph' 
    case 'low_stretch_tree' % 2^k vertices on each side (k>=1)
        if isfield(param,'k')
            k=param.k;
        elseif isfield(param,'N')
            k=round(log(sqrt(param.N))/log(2));
        else
            k=5;
        end
        G=gsp_create_low_stretch_tree(k);
    otherwise 
        error('Unexpected graph type');
end

G.graph_type=graph_type;

% Create degree matrix
G.D=diag(sum(G.W));

% Create graph Laplacian
if ~isfield(param,'lap_type')
    lap_type='combinatorial';
else 
    lap_type=param.lap_type;
end

switch lap_type
    case 'combinatorial'
        G.L=G.D-G.W;
        G.lap_type='combinatorial';
    case 'normalized'
        G.L=G.D-G.W;
        factor=diag(diag(G.D).^(-.5));
        G.L=factor*G.L*factor;
        G.lap_type='normalized';
    otherwise
        G.lap_type='none';
end

% Create unweighted adjacency matrix
thresh=1e-8;
G.A=abs(G.W)>thresh;

% Assign default plotting parameters
if ~isfield(G,'edge_width')
    G.edge_width=1;
end
if ~isfield(G,'edge_color')
    G.edge_color=[255,88,41]/255;
end
if ~isfield(G,'edge_style')
    G.edge_style='-';
end
if ~isfield(G,'vertex_size')
    G.vertex_size=100;
end
if ~isfield(G,'vertex_color')
    G.vertex_color='b';
end
if ~isfield(G,'vertex_edge_color')
    G.vertex_edge_color='k';
end
