function [G]=gsp_create_ring_graph(N,k)
%GSP_CREATE_RING_GRAPH  Initialize a ring graph
%   Usage:  G=gsp_create_ring_graph(N);
%
%   Input parameters:
%         N     : Number of vertices.
%         k     : Number of neighbors in each direction
%   Output parameters:
%         G     : Graph structure.
%
%   'gsp_create_ring_graph(N)' initializes a graph structure containing
%   the weighted adjacency matrix (G.W), the number of vertices (G.N), the 
%   plotting coordinates (G.coords), and the plotting coordinate limits 
%   (G.coord_limits) of a ring graph with N vertices. Each vertex in the 
%   ring has 2k neighbors (maximum value of k is N/2). The edge 
%   weights are all equal to 1.
%
%   Url: http://gspbox.sourceforge.net/doc/graphs/specific_graphs/gsp_create_ring_graph.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE: 

G.N=N;

if k>N/2
    error('Too many neighbors requested');
end

% Create weighted adjancency matrix
if k==N/2
    num_edges=N*(k-1)+N/2;
else
    num_edges=N*k;
end
i_inds=zeros(1,2*num_edges);
j_inds=zeros(1,2*num_edges);

all_inds=1:N;
for i=1:min(k,floor((N-1)/2))
   i_inds((i-1)*2*N+1:(i-1)*2*N+N)=all_inds;
   j_inds((i-1)*2*N+1:(i-1)*2*N+N)=1+mod(all_inds-1+i,N);
   i_inds((i-1)*2*N+N+1:i*2*N)=1+mod(all_inds-1+i,N);
   j_inds((i-1)*2*N+N+1:i*2*N)=all_inds;
end

if k==N/2
   i_inds(2*N*(k-1)+1:2*N*(k-1)+N)=all_inds;
   j_inds(2*N*(k-1)+1:2*N*(k-1)+N)=1+mod(all_inds-1+k,N);
end

G.W=sparse(i_inds,j_inds,ones(1,length(i_inds)),N,N);

% Create coordinates
G.coords=[(cos((0:N-1)*(2*pi)/N))',(sin((0:N-1)*(2*pi)/N))'];
G.coord_limits=[-1,-1;1,1];

