function [G]=gsp_create_sensor_network(N)
%GSP_CREATE_SENSOR_NETWORK  Initialize a sensor network
%   Usage:  G=gsp_create_sensor_network(N);
%
%   Input parameters:
%         N     : Number of vertices.
%   Output parameters:
%         G     : Graph structure.
%
%   'gsp_create_sensor_network(N)' initializes a graph structure containing
%   the weighted adjacency matrix (G.W), the number of vertices (G.N), the 
%   plotting coordinates (G.coords), and the plotting coordinate limits 
%   (G.coord_limits) of a random sensor network with N vertices. The 
%   sensors are placed randomly in the unit square, and edges are placed 
%   between any sensors within a fixed radius of each other. The edge 
%   weights are assigned via a thresholded Gaussian kernel. The sensor 
%   network will be connected for N=500 or N=64. 
%
%   Requires: distanz
%
%   Url: http://gspbox.sourceforge.net/doc/graphs/specific_graphs/gsp_create_sensor_network.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE: 

randn('seed', 18); rand('seed', 18);

G.N=N;

% Generate sensor locations
Xcoords = rand(N,1);
Ycoords = rand(N,1);
G.coords = [Xcoords,Ycoords];
G.coord_limits = [0,0;1,1];

% Create weighted adjancency matrix
target_dist_cutoff = -.125*N/436.075+.2183;
T = .6; 
s = sqrt(-target_dist_cutoff^2/(2*log(T)));
d = gsp_distanz(G.coords'); 
G.W = exp(-d.^2/(2*s^2)); 
G.W(G.W<T) = 0; % Thresholding to have sparse matrix
G.W=G.W-diag(diag(G.W));
G.W=sparse(G.W);

