function [G]=gsp_full_eigen(G)
%GSP_FULL_EIGEN  Compute a full eigendecomposition of the graph Laplacian
%   Usage:  G=gsp_full_eigen(G);
%
%   Input parameters:
%         G          : Graph structure.
%   Output parameters:
%         G          : Graph structure.
%
%   'gsp_full_eigen(G)' computes a full eigendecomposition of the graph
%   Laplacian G.L:
%
%      L = U Lambda U* 
%
%   where Lambda is a diagonal matrix of the Laplacian eigenvalues. 
%   G.E is a column vector of length G.N containing the Laplacian eigenvalues.  
% 
%   See also:  
%
%   Demos:  
% 
%
%
%   Url: http://gspbox.sourceforge.net/doc/operators/gsp_full_eigen.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  

if ( isfield(G,'E') || isfield(G,'U') )
    warning('Laplacian eigenvalues or eigenvectors are already associated with this graph');
end

if G.N > 3000
    warning('Performing full eigendecomposition of a large matrix may take some time...'); 
end
    
if ( strcmp(G.graph_type,'ring')==1 && mod(G.N,2)==0 )
    U=dftmtx(G.N)/sqrt(G.N);
    E=(2-2*cos(2*pi*(0:G.N-1)/G.N))';
    [G.E inds]=sort(E,'ascend');
    G.U=U(:,inds');
else
    if ~isfield(G,'L')
        error('Graph Laplacian is not provided.');
    end
    
    % Compute and all eigenvalues and eigenvectors 
    [eigenvectors,eigenvalues,~]=svd(full(G.L));

    % Sort eigenvectors and eigenvalues
    [G.E,inds] = sort(diag(eigenvalues),'ascend');
    eigenvectors=eigenvectors(:,inds');
    
    % Set first component of each eigenvector to be nonnegative
    signs=sign(eigenvectors(1,:));
    signs(signs==0)=1;
    G.U = eigenvectors*diag(signs);
end

G.lmax=max(G.E);
