function [coeffs,varargout]=gsp_translates_analysis(G,filters,signal,varargin)
%GSP_TRANSLATES_ANALYSIS Compute a full eigendecomposition of the graph Laplacian
%   Usage:  coeffs=gsp_translates_analysis(G,filters,signal);
%           coeffs=gsp_translates_analysis(G,filters,signal,param);
%           [coeffs,analysis_matrix]=gsp_translates_analysis(G,filters,signal,param);
%           [coeffs,analysis_matrix,G]=gsp_translates_analysis(G,filters,signal,param);
%
%   Input parameters:
%         G                : Graph structure.
%         filters          : Set of spectral graph filters.
%         signal           : graph signal to analyze.
%   Output parameters:
%         coeffs           : Transform coefficients (reshaped into an N x num_filters matrix).
%         analysis_matrix  : The analysis matrix with each column representing one atom (only computed when param.use_exact=1).
%   Additional parameters:
%         param.use_exact  : To use exact graph spectral filtering instead of the Chebyshev approximation.
%         param.cheb_order : Degree of the Chebyshev approximation (default=30).
%
%   'gsp_translates_analysis(G,filters,signal)' computes the transform
%   coefficients of a signal f, where the atoms of the transform
%   dictionary are generalized translations of each graph spectral filter
%   to each vertex on the graph.
%
%      coeffs = D* f 
%
%   where the columns of {cal D} are g_{i,m}=T_i g_m, and T_i is a generalized translation operator applied to each filter hat{g}_m(cdot).
% 
%   See also:  
%
%   Demos:  
% 
%
%   Url: http://gspbox.sourceforge.net/doc/operators/gsp_translates_analysis.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
% Read input parameters
if nargin>3
    param=varargin{1};
else
    param=0;
end

num_filters=length(filters);

if ~isfield(param,'use_exact');
    use_exact=0;
else
    use_exact=param.use_exact;
end

if use_exact 
    varargout=cell(2,1);
    if ( ~isfield(G,'E') || ~isfield(G,'U') )
        G=gsp_full_eigen(G);
        varargout{2}=G;
    end
    
    analysis_matrix=zeros(G.N*num_filters,G.N);

    for i=1:num_filters
        analysis_matrix((i-1)*G.N+1:i*G.N,:)=G.U*diag(filters{i}(G.E))*G.U';
    end

    coeffs=analysis_matrix*signal;
    varargout{1}=analysis_matrix;

else % use Chebyshev approximation method

    if ~isfield(param,'cheb_order');
        cheb_order=30;
    else
        cheb_order=param.cheb_order;
    end
    
    if ~isfield(G,'lmax');
        G.lmax=sgwt_rough_lmax(G.L);
    end
    
    coeffs=zeros(num_filters*G.N,1);
    for i=1:num_filters
        cheb_coeffs=sgwt_cheby_coeff(filters{i},cheb_order,cheb_order+1,[0,G.lmax]);
        coeffs((i-1)*G.N+1:i*G.N)=sgwt_cheby_op(signal,G.L,cheb_coeffs,[0,G.lmax]);
    end

end

coeffs=reshape(coeffs,G.N,num_filters);

