function [filter_data,test_sum]=gsp_plot_filters(filters,range,varargin)
%GSP_PLOT_FILTERS  Plot a system of filters
%   Usage:  [filter_data,test_sum]=gsp_plot_filters(filters,range);
%           [filter_data,test_sum]=gsp_plot_filters(filters,range,param);
%
%   Input parameters:
%         filters                  : Cell array with each element containing one graph spectral filter.
%         range                    : Range of values at which to evaluate the filters.
%         param                    : Optional variable containing additional parameters.
%   Output parameters:
%         filter_data              : The values of the filters at the sampled frequencies.
%         test_sum                 : The sum of squared magnitudes of the filters at the sampled frequencies.
%
%   Additional parameters:
%         param.line_width         : Width of the filter plots.
%         param.font_size          : Font size of axis labels.
%         param.dlambda            : Distance between points at which the filters are evaluated.
%         param.x_tic              : Distance between x-tick labels.
%         param.y_tic              : Distance between y-tick labels.
%         param.minor_tick         : To show minor tick marks.
%         param.title              : Figure title.
%         param.lambda_full        : To plot black X marks at all eigenvalues.
%         param.lambda_highlights  : To plot red X marks at highlight eigenvalues.
%         param.x_width            : Width of X marks for the eigenvalues.
%         param.x_size             : Size of X marks for the eigenvalues.
%         param.show_sum_squares   : To plot an extra line showing the sum of the squared magnitudes of the filters.
%         param.colors_rgb         : To specify the line colors.
%         param.yrange             : To specify a range for the y axis.
%
%   Url: http://gspbox.sourceforge.net/doc/plotting/gsp_plot_filters.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%
%   'gsp_plot_filters(filters,range)' plots a system of graph spectral filters.
%
%   See also:  
%
%   Demos:  
%
%   References:

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
% Read input parameters
if nargin>2
    param=varargin{1};
else
    param=0;
end

if length(range)==1
    range=[0,range];
end

if isfield(param,'dlambda')
    dlambda=param.dlambda;
else
    dlambda = max(.005,(range(2)-range(1))/20000);
end

if isfield(param,'line_width')
    line_width=param.line_width;
else
    line_width=4;
end

if isfield(param,'x_width')
    x_width=param.x_width;
else
    x_width=4;
end

if isfield(param,'x_size')
    x_size=param.x_size;
else
    x_size=16;
end

if isfield(param,'show_sum_squares')
    show_sum=param.show_sum_squares;
else
    show_sum=1;
end

num_filters=length(filters);

lambdas=(range(1):dlambda:range(2))';

figure;
if isfield(param,'colors_rgb');
    set(gca, 'ColorOrder', param.colors_rgb);
end
hold on;
filter_data=zeros(length(lambdas),num_filters);
for k=1:num_filters
    filter_data(:,k)=filters{k}(lambdas);
end
plot(lambdas,filter_data,'LineWidth',line_width);

if isfield(param,'lambda_full')
    plot(param.lambda_full,zeros(length(param.lambda_full),1),'xk','LineWidth',x_width,'MarkerSize',x_size);
end

if isfield(param,'lambda_highlights')
    plot(param.lambda_highlights,zeros(length(param.lambda_highlights),1),'xr','LineWidth',x_width,'MarkerSize',x_size);
end

if show_sum
    test_sum=sum(filter_data.^2,2); 
    plot(lambdas,test_sum,'k','LineWidth',line_width);
else 
    test_sum=NaN;
end

if isfield(param,'title')
    set(gca,'FontSize',18);
    title(param.title);
end

if isfield(param,'font_size')
    set(gca,'FontSize',param.font_size);
else
    set(gca,'FontSize',24);
end



if isfield(param,'minor_tick')
    if param.minor_tick
        set(gca,'XMinorTick','on','YMinorTick','on');
    end
end

if ~isfield(param,'x_tic')
    x_tic=max(1,ceil((range(2)-range(1))/10));
else 
    x_tic=param.x_tic;
end

if ~isfield(param,'y_tic')
    y_tic=.25;
else
    y_tic=param.y_tic;
end


box on;
xlim(range);
if isfield(param,'yrange')
    yrange=param.yrange;
    ylim(yrange);
    set(gca,'YTick',yrange(1):y_tic:yrange(2));
end
set(gca,'XMinorTick','on','YMinorTick','on');
set(gca,'XTick',range(1):x_tic:range(2));



end



