function gsp_plot_graph_signal(G,signal,varargin)
%GSP_PLOT_GRAPH_SIGNAL  Plot a graph signal in 2D or 3D
%   Usage:  gsp_plot_graph(G,signal);
%           gsp_plot_graph(G,signal,param);
%
%   Input parameters:
%         G          : Graph structure.
%         signal     : Graph signal.
%         param      : Optional variable containing additional parameters.
%   Output parameters:
%         none
%
%   Additional parameters:
%         param.show_edges     : Set to 0 to only draw the vertices.
%         param.cp             : Camera position for a 3D graph.
%         param.pt_size        : Size of circle representing each signal component.
%         param.use_colorbar   : Set to 0 to not show the colorbar
%         param.climits        : Limits of the colorbar
%         param.use_old_figure : Do not create a new figure
%
%   'gsp_plot_graph(G)' plots a graph signal in 2D or 3D, using the adjacency 
%   matrix (G.A), the plotting coordinates (G.coords), the 
%   coordinate limits (G.coord_limits), the edge width (G.edge_width), the 
%   edge color (G.edge_color), the edge style (G.edge_style), and the 
%   vertex size (G_vertex_size).
%
%   See also:  
%
%   Demos:  
%
%   Requires: gplot23D
%
%   Url: http://gspbox.sourceforge.net/doc/plotting/gsp_plot_graph_signal.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
% Read input parameters
if nargin>2
    param=varargin{1};
else
    param=0;
end

if ~isfield(param,'show_edges')
    if nnz(G.A) > 20000
        show_edges=0;
    else
        show_edges=1;
    end
else 
    show_edges=param.show_edges;
end

if ~isfield(param,'pt_size')
    pt_size=500;
else 
    pt_size=param.pt_size;
end

if ~isfield(param,'use_colorbar')
    use_colorbar=1;
else 
    use_colorbar=param.use_colorbar;
end

if isfield(param,'climits')
    climits=param.climits;
else 
    cmin=min(0,1.01*min(signal));
    cmax=max(0,1.01*max(signal));
    climits=[cmin,cmax];
end

if isfield(param,'use_old_figure')
    if param.use_old_figure==0
        figure;
    end
else
    figure;
end
hold on;

if show_edges
    gplot23D(G.A,G.coords,G.edge_style,'LineWidth',G.edge_width,'Color',G.edge_color); 
end

if size(G.coords,2)==2
    scatter(G.coords(:,1),G.coords(:,2),pt_size,signal,'.');
    xlim([G.coord_limits(1,1) G.coord_limits(2,1)]);
    ylim([G.coord_limits(1,2) G.coord_limits(2,2)]);
elseif size(G.coords,2)==3
    scatter3(G.coords(:,1),G.coords(:,2),G.coords(:,3),pt_size,signal,'.');
    xlim([G.coord_limits(1,1) G.coord_limits(2,1)]);
    ylim([G.coord_limits(1,2) G.coord_limits(2,2)]);
    zlim([G.coord_limits(1,3) G.coord_limits(2,3)]);
end

if size(G.coords,2)==3
    if ~isfield(param,'cp')
        cp=[-1.4,-16.9,3.4];
    else 
        cp=param.cp;
    end
    set(gca,'CameraPosition',cp);
end

caxis(climits);

if use_colorbar
    colorbar;
end
axis off;

