function gsp_plot_warp_fn(warp_fn,range,varargin)
%GSP_PLOT_WARP_FN  Plot a warping function
%   Usage:  gsp_plot_warp_fn(warp_fn,range);
%           gsp_plot_warp_fn(warp_fn,range,param);
%
%   Input parameters:
%         warp_fn              : Single warp function or a cell of warp functions.
%         range                : Range of values at which to evaluate the warp function.
%         param                : Optional variable containing additional parameters.
%   Output parameters:
%         none
%
%   Additional parameters:
%         param.draw_ref_line  : Include a diagonal reference line.
%         param.line_width     : Width of warping function plots.
%         param.font_size      : Font size of axis labels.
%         param.x_len          : Distance between points at which warping function is evaluated.
%         param.x_tic          : Distance between x-tic labels.
%         param.y_tic          : Distance between y-tic labels.
%         param.use_old_figure : Do not create a new figure
%
%   'gsp_plot_warp_fn(warp_fn,range)' plots one or more warping functions.
%   If plotting multiple warping functions, the spectrum range should be
%   the same for all of the warp functions.
%
%   See also:  
%
%   Demos:  
%
%
%   Url: http://gspbox.sourceforge.net/doc/plotting/gsp_plot_warp_fn.php

% Copyright (C) 2013 David I Shuman, Nathanael Perraudin.
% This file is part of GSPbox version 0.0.1
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

%   AUTHOR : David I Shuman.
%   TESTING: 
%   REFERENCE:
  
% Read input parameters
if nargin>2
    param=varargin{1};
else
    param=0;
end

if length(range)==1
    range=[0,range];
end

if ~isfield(param,'draw_ref_line')
    draw_ref_line=1;
else 
    draw_ref_line=param.draw_ref_line;
end

if ~isfield(param,'line_width')
    line_width=4;
else 
    line_width=param.line_width;
end

if ~isfield(param,'font_size')
    font_size=24;
else 
    font_size=param.font_size;
end

if ~isfield(param,'x_len')
    x_len=max(.01,(range(2)-range(1))/1000);
else 
    x_len=param.x_len;
end

if ~isfield(param,'x_tic')
    x_tic=max(1,ceil((range(2)-range(1))/10));
else 
    x_tic=param.x_tic;
end

if ~isfield(param,'y_tic')
    y_tic=.25;
else
    y_tic=param.y_tic;
end

warp_colors=repmat(['r','b','g','y','m'],1,10);

xx=range(1):x_len:range(2);
if isfield(param,'use_old_figure')
    if param.use_old_figure==0
        figure;
    end
else
    figure;
end
hold on;
num_warp_fn=length(warp_fn);
if num_warp_fn>50
    error('Too many warp functions to plot on one graph');
end
if num_warp_fn>1
    yrange=[0,0];
    for i=1:num_warp_fn
        plot(xx,warp_fn{i}(xx),warp_colors(i),'LineWidth',line_width);
        yrange(1)=min(yrange(1),warp_fn{i}(range(1)));
        yrange(2)=max(yrange(1),warp_fn{i}(range(2)));
    end
else
    plot(xx,warp_fn(xx),'r','LineWidth',line_width);
    yrange=warp_fn(range);
end
if draw_ref_line
    plot(xx,xx/range(2),'k:','LineWidth',1);
end
box on;
set(gca,'FontSize',font_size);
ylim(yrange);
xlim(range);
set(gca,'XMinorTick','on','YMinorTick','on');
set(gca,'XTick',range(1):x_tic:range(2));
set(gca,'YTick',yrange(1):y_tic:yrange(2));

